dojo.provide("com.ibm.pb.data.NavigationPrimer");

dojo.require("com.ibm.pb.logging.Loggable");

dojo.declare("com.ibm.pb.data.NavigationPrimer", com.ibm.pb.logging.Loggable, {
	
	LEVEL_ATTR: "data-nm-level",
	PRIMED_ATTR: "data-nm-primed",
	END: {},
	
	selPath: null,	// Array of selection path starting from root going down towards selected node
	map: null,		// Object map of priming node data with keys as node ids
	
	constructor: function() {
		this.selPath = [];
		this.map = {};
	},
	
	parse: function(domNode) {
		var mname = "parse", lgr = this.LOGGER;
		lgr.entering(mname, [domNode]);
		domNode = dojo.byId(domNode);
		var level = dojo.attr(domNode, this.LEVEL_ATTR);
		if(level !== null && level !== undefined) {
			lgr.trace(mname, "Priming the parse root node itself");
			this._prime(domNode);
		}
		// now look for descendants that may have the level attribute defined
		var levelNodes = dojo.query("[" + this.LEVEL_ATTR + "]", domNode);
		if(levelNodes.length > 0) {
			levelNodes.forEach(this._prime, this);
		}
		else {
			lgr.trace(mname, "No level nodes detected in descendant nodes");
			lgr.exiting(mname, [null]);
			return;
		}
		
		lgr.exiting(mname, [domNode]);
	},
	_prime: function(domNode) {
		var mname = "_prime", lgr = this.LOGGER;
		lgr.entering(mname, [domNode]);
		var level = dojo.attr(domNode, this.LEVEL_ATTR);
		
		var nodeData = this.extractData(domNode);
		if(nodeData) {
			this.addItem(nodeData);
			if(!this.selPath[level]) {
				lgr.trace(mname, "No previous selection path data at level ${0}", [level]);
			}
		}
		else {
			if(this.selPath[level]) {
				nodeData = this.selPath[level];
				lgr.trace(mname, "No data provided for selection path data, using previously known", []);
			}
			else {
				lgr.trace(mname, "No data provided for selection path data, and no previous data to use", []);
				lgr.exiting(mname, []);
				return;
			}
		}
		nodeData = this.byId(nodeData.id);
		this.selPath[level] = nodeData;
		lgr.trace(mname, "Priming selection path data at level ${0}", [level]);
		var childNodes = dojo.query("[" + this.PRIMED_ATTR + "]", domNode);
		if(childNodes.length > 0) {
			lgr.trace(mname, "Found children of ${0}", [nodeData.id]);
			if(!nodeData.children) {
				nodeData.children = [];
			}
			childNodes.forEach(function(childDomNode){
				var childData = this.extractData(childDomNode);
				if(childData) {
					lgr.trace(mname, "Adding child ${0} of ${1}", [childData.id, nodeData.id]);
					this.addItem(childData);
					childData = this.byId(childData.id);
					nodeData.children.push(childData);
				}
			}, this);
			nodeData.children.push(this.END);
		}
		else {
			lgr.trace(mname, "Found NO children of ${0}", [nodeData.id]);
		}
		lgr.exiting(mname, [nodeData]);
	},
	addItem: function(data){
		var mname = "addItem", lgr = this.LOGGER;
		if(data && data.id) {
			if(this.map[data.id]) {
				// data already primed... mix in any additional attributes
				lgr.trace(mname, "Adding properties ${0} for ${1} to existing data", [data, data.id]);
				dojo.mixin(this.map[data.id], data);
			}
			else {
				// data not primed yet
				lgr.trace(mname, "Adding new data ${0} for ${1}", [data, data.id]);
				this.map[data.id] = data;
			}
		}
	},
	byId: function(id) {
		return this.map[id];
	},
	_normalizeType: function(type) {
		type = type ? type.toLowerCase() : "page";
		switch(type) {
			case "internalurl":
			case "externalurl":
				type = "url";
				break;
			case "staticpage":
				type = "page";
				break;
		}
		return type;
	},
	extractData: function(domNode) {
		var mname = "extractData", lgr = this.LOGGER;
		var data = dojo.attr(domNode, this.PRIMED_ATTR);
		if(data) {
			data = dojo.fromJson(data);
			data.locale = ibmConfig.locale;
			data.type = this._normalizeType(data.type);
			lgr.trace(mname, "Extracted data ${0}", [data]);
		}
		else {
			data = null;
		}
		return data;
	},
	finalize: function() {
		var mname = "finalize", lgr = this.LOGGER;
		for(var i=this.selPath.length-1; i>=0; i--) {
			if(i > 0) {
				if(this.selPath[i-1]){
					if(!this.selPath[i-1].children) {
						this.selPath[i-1].children = [this.selPath[i]];
					}
					else if(this.selPath[i-1].children.length == 0) {
						// pushing the END marker onto the beginning along with other nodes
						// specifies that this set of children is incomplete, but that it
						// doesn't start at the beginning; hence the system will get all
						// children from the model if getChildren is called on this parent
						this.selPath[i-1].children.push(this.END, this.selPath[i]);
					}
				}
			}
		}
		var nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		if(lgr.isLoggable(com.ibm.mashups.enabler.logging.LogLevel.TRACE)) {
			lgr.trace(mname, "Priming navigation model ${0}", [this]);
			lgr.trace(mname, "Priming data:\n  ${0}", [dojo.toJson(this.selPath[0], true)]);
		}
		var firstDefined;
		for (var i = 0; i < this.selPath.length; i++){
			if (firstDefined == null && this.selPath[i]){
				firstDefined = i;
			}
		}
		nm.setPrimingData(this.selPath[firstDefined]);
	}
});